from typing import Optional, Tuple
from dsc_labs.connectors.aws.base import AWSConnector


class IoTDataPlaneConnector(AWSConnector):
    def __init__(self,
                 aws_access_key_id: str = None,
                 aws_secret_access_key: str = None,
                 region_name: str = None) -> None:
        super().__init__('iot-data',
                         aws_access_key_id=aws_access_key_id,
                         aws_secret_access_key=aws_secret_access_key,
                         region_name=region_name)

    def publish(self,
                topic: str,
                payload: bytes,
                qos: int = 1,
                retain: bool = False) -> Tuple[bool, str]:
        """
        Publishes an MQTT message
        :param topic: The name of the MQTT topic
        :param payload: The message body. MQTT accepts text, binary, and empty (null) message payloads
        :param qos: The Quality of Service (QoS) level
        :param retain: A Boolean value that determines whether to set the RETAIN flag when the message is published
        """
        try:
            self.client.publish(topic=topic, qos=qos, retain=retain, payload=payload)
            self.log.info('event=publish-mqtt-message-success topic={}'.format(topic))
            return True, ''
        except BaseException as e:
            error_msg = str(e)
            self.log.error('event=publish-mqtt-message-failure topic={} message="{}"'.format(topic, error_msg))
        return False, error_msg

    def get_retained_message(self, topic: str) -> Tuple[Optional[dict], str]:
        """
        Gets the details of a single retained message for the specified topic
        :param topic: The topic name of the retained message to retrieve
        :return: MQTT message
        """
        try:
            response = self.client.get_retained_message(
                topic=topic
            )
            self.log.info('event=get-retained-message-success topic={}'.format(topic))
            return response, ''
        except BaseException as e:
            error_msg = str(e)
            self.log.error('event=get-retained-message-failure topic={} message="{}"'.format(topic, error_msg))
        return None, error_msg



#! /usr/bin/env python3

import logging

from bugsnag.handlers import BugsnagHandler


class DSCLabsLogger:
    """
    Initialize and return a logger instance

    Optionally, adds a bugsnag handler. The handler reports log events
    for error levels ERROR and above.
    """
    def __init__(self, default_handler=False) -> None:
        log_format = ('time="%(asctime)-15s" module=%(module)s '
                      'level=%(levelname)-7s %(message)s')
        self.default_handler = default_handler
        logging.basicConfig(format=log_format)
        # For lambda functions
        if len(logging.getLogger().handlers):
            handler = logging.getLogger().handlers[0]
            handler.setFormatter(logging.Formatter(log_format))
        # Enable INFO logs for dsc_labs libs and apps only
        self.logger_instance = logging.getLogger(__name__)
        self.logger_instance.setLevel(logging.INFO)
        logging.getLogger('dsc_labs').setLevel(logging.INFO)

    @staticmethod
    def get_bugsnag_handler() -> BugsnagHandler:
        handler = BugsnagHandler()
        # Notify only ERROR levels and above.
        handler.setLevel(logging.ERROR)
        return handler

    def get_logger(self) -> logging.Logger:
        if self.default_handler:
            return self.logger_instance
        bugsnag_handler = self.get_bugsnag_handler()
        self.logger_instance.addHandler(bugsnag_handler)
        return self.logger_instance


class DSCLabsLoggerMixin:
    def __init__(self) -> None:
        mll = DSCLabsLogger(default_handler=True)
        self.log = mll.get_logger()


class DSCLabsAPILoggerMixin:
    def __init__(self) -> None:
        mll = DSCLabsLogger()
        self.log = mll.get_logger()



#!/usr/bin/env python3
import argparse


from pprint import pprint

from dsc_labs.libs.logger import DSCLabsLogger
from dsc_labs.utils.aws_config_manager import AWSConfigManager

li = DSCLabsLogger(default_handler=True)
logger = li.get_logger()


def run(service, op, opts):
    try:
        cm = AWSConfigManager(service, opts)
        if op == 'get':
            value = cm.get_param()
            pprint(value)
        else:
            resp = cm.put_param()
            pprint(resp)
    except Exception as ex:
        logger.error('Failed: operation="%s" service="%s" error="%s"',
                     op, service, ex)


def main():
    parser = argparse.ArgumentParser(description='DSC-Labs services config helper')
    parser.add_argument(
        '-d', '--deployment',
        default='dev',
        help='The deployment type dev/prod'
    )
    parser.add_argument(
        '-r', '--region',
        default='us-east-1',
        help='The AWS region to use'
    )
    parser.add_argument(
        '-s', '--service',
        required=True,
        help='The service name for the config'
    )
    parser.add_argument(
        '-o', '--operation',
        required=True,
        choices=['get', 'put'],
        help='The desired operation: one of [get, put]'
    )
    parser.add_argument(
        '-c', '--config',
        help='The config file path when operation is put'
    )
    args = parser.parse_args()
    opts = {
        'deployment': args.deployment,
        'region': args.region,
        'config': args.config
    }
    run(args.service, args.operation, opts)



#!/usr/bin/env python3

import os
import yaml
import boto3


from yaml.parser import ParserError

from dsc_labs.libs.logger import DSCLabsLogger


class AWSConfigManager:
    def __init__(self, service, opts):
        li = DSCLabsLogger(default_handler=True)
        self.log = li.get_logger()
        self.deployment = opts['deployment']
        self.service = service
        self.config = opts.get('config')
        self.client = boto3.client('ssm', region_name=opts['region'])
        self.parameter_path = 'config.dsc-labs.%s.%s-service' % (
            self.deployment, self.service
        )

    def get_param(self):
        parameter = self.client.get_parameter(
            Name=self.parameter_path,
            WithDecryption=True
        )
        value = parameter.get('Parameter', {}).get('Value', '')
        if not value:
            raise RuntimeError('Unable to get the specified parameter from AWS')
        return yaml.safe_load(value)

    def is_config_readable(self):
        return os.path.exists(self.config) and os.access(self.config, os.R_OK)

    @staticmethod
    def is_config_valid(config_val):
        try:
            yaml.safe_load(config_val)
            return True
        except ParserError:
            return False

    def put_param(self):
        if not self.config:
            raise RuntimeError('Need a config file for put operation')
        if not self.is_config_readable():
            raise IOError('Unable to read config from file: %s' % self.config)
        config_val = open(self.config).read()
        if not self.is_config_valid(config_val):
            raise RuntimeError('Invalid config file: %s' % self.config)
        resp = self.client.put_parameter(
            Name=self.parameter_path,
            Description='deployment=%s service_name=%s-service' % (
                self.deployment,
                self.service
            ),
            Value=config_val,
            Overwrite=True,
            Type='SecureString',
            Tier='Standard'
        )
        status = resp.get('ResponseMetadata', {}).get('HTTPStatusCode')
        if status != 200:
            raise RuntimeError('Failed to update config for %s' % self.service)
        return status



#!/usr/bin/env python3

import os
import yaml
from typing import Any, Dict

from dsc_labs.utils.aws_config_manager import AWSConfigManager

DEPLOYMENT = os.environ.get('DEPLOYMENT') or 'dev'
LOCAL_CONFIG = os.environ.get('LOCAL_CONFIG')
REGION = os.environ.get('AWS_REGION') or 'us-east-1'


class DSCLabsAppConfig:
    """
    Load and return service configuration

    If ``LOCAL_CONFIG`` is set, use it to read config
    Else read config from AWS Config Manager
    """
    def __init__(self, service_name: str) -> None:
        self.service_name = service_name

    def get_config(self) -> Dict[str, Any]:
        if LOCAL_CONFIG and os.path.exists(LOCAL_CONFIG):
            return self.get_local_config()
        return self.get_aws_config()

    @staticmethod
    def get_local_config() -> Dict[str, Any]:
        with open(LOCAL_CONFIG) as lc:
            return yaml.safe_load(lc.read())

    def get_aws_config(self) -> Dict[str, Any]:
        opts = {
            'deployment': DEPLOYMENT,
            'region': REGION
        }
        cm = AWSConfigManager(self.service_name, opts)
        return cm.get_param()



import json
import yaml

from typing import Optional
from logging import INFO
from datetime import datetime, timedelta

from dsc_labs.libs.config import DSCLabsAppConfig
from dsc_labs.libs.logger import DSCLabsLogger
from dsc_labs.libs.bugsnag import init_bugsnag


def get_common_config(service_name: str = "") -> dict:
    """
    Initialize common configurations.
    :return: A dict containing the whole common config data.
    """
    mlc = DSCLabsAppConfig(service_name)
    common_config = mlc.get_config()

    return common_config


def datetime_to_utc(timestamp: datetime) -> str:
    """
    Convert timestamp (datetime object) to UTC time
    :param timestamp: datetime
    :return: UTC time as string with iso format
    """
    return f'{timestamp.isoformat(timespec="milliseconds")}Z'


def current_utc_timestamp() -> str:
    """
    Get current UTC timestamp
    :return: The current UTC timestamp.
    """
    return datetime_to_utc(datetime.utcnow())


def current_utc_timestamp_filename(datetime_format: str = '%Y-%m-%d_%H%M%S%f',
                                   delta_time: int = 0) -> str:
    """
    Get current utc timestamp (millisecond) as filename
    :param datetime_format: datetime format, should use dash and underscore instead of whitespace and colon
    :param delta_time: Delta time with each timezone. Ex: VN - timezone=7 (UTC+7)
    :return: Timestamp with filename format
    """
    now = datetime.utcnow() + timedelta(hours=delta_time)
    return now.strftime(datetime_format)


def int_to_utc(timestamp: int) -> str:
    """
    Convert timestamp (int) to UTC timestamp
    :param timestamp: Timestamp as int
    :return: The current UTC timestamp as string
    """
    return datetime_to_utc(datetime.fromtimestamp(timestamp/1e3))


def init_logger():
    """
    Initialize bugsnag and global logger object
    """
    mll = DSCLabsLogger()
    logger = mll.get_logger()
    logger.setLevel(INFO)
    return logger


def init_bugsnag_connector(config: dict, app_home: str) -> None:
    """
    Initialize Bugsnag API connector.
    """
    init_bugsnag(
        api_key=config.get('bugsnag', {}).get('api_key', ''),
        app_dir=app_home
    )


def read_json(file_path: str) -> Optional[dict]:
    """
    Read data from a local json file
    :param file_path: Path to json file
    :return: Dictionary if success, otherwise return None
    """
    try:
        with open(file_path, 'r') as file:
            data = json.load(file)
        return data
    except Exception:
        return None


def read_yaml(file_path: str) -> Optional[dict]:
    """
    Read data from a local yaml file
    :param file_path: Path to yaml file
    :return: Dictionary if success, otherwise return None
    """
    with open(file_path, 'r') as file:
        try:
            return yaml.safe_load(file.read())
        except yaml.YAMLError:
            return None



#!/usr/bin/env python3
from setuptools import setup, find_packages
from dsc_labs.version import __prog__, __version__

setup(
    name=__prog__,
    version=__version__,
    description='Common libraries for DSC projects/services',
    author='Danh Nguyen.T',
    author_email='ntdanh1@tma.com.vn',
    url='https://gitlab.tma.com.vn/data-science-group/dsc-labs/-/tree/main/dsc_packages/common/python',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    keywords=['pip', 'dsc_labs', 'common'],
    install_requires=[
        'boto3>=1.17.49',
        'botocore>=1.20.49',
        'PyYAML>=5.1.2',
        'bugsnag>=3.6.0',
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            '{0}-config=dsc_labs.cli.config:main'.format(__prog__)
        ]
    }
)



import json

from typing import Union, Optional, Tuple
from botocore.exceptions import ClientError
from dsc_labs.connectors.aws.base import AWSConnector


class S3Connector(AWSConnector):
    def __init__(self, aws_access_key_id: str = None, aws_secret_access_key: str = None) -> None:
        super().__init__('s3',
                         aws_access_key_id=aws_access_key_id,
                         aws_secret_access_key=aws_secret_access_key)

    def download_object(self, bucket: str, key: str, local_path: str = '') -> Tuple[bool, str]:
        """
        Downloading a file from s3 to file_path
        :param bucket: S3 bucket name
        :param key: S3 object key
        :param local_path: path to save file
        :return: True if it is successful, False if it encountered error
        """

        try:
            self.client.download_file(Bucket=bucket, Key=key, Filename=local_path)
            self.log.info('event=download-s3-object-success bucket={} key={}'.format(bucket, key))
            return True, ''
        except ClientError as e:
            if e.response['Error']['Code'] == '404':
                error_msg = 'The object key does not exist.'
            elif e.response['Error']['Code'] == '403':
                error_msg = 'An error occurred (403) when calling the HeadObject operation: Forbidden.'
            else:
                error_msg = str(e)
            self.log.error('event=download-s3-object-failure bucket={} key={} message="{}"'
                           .format(bucket, key, error_msg))
        except BaseException as e:
            error_msg = str(e)
            self.log.error('event=download-s3-object-failure bucket={} key={}'.format(bucket, key))
        return False, error_msg

    def get_object(self, bucket: str, key: str) -> Tuple[Optional[dict], str]:
        """
        Get S3 object
        :param bucket: S3 bucket name
        :param key: S3 object key
        :return: Object if success, otherwise return None
        """
        try:
            s3_object = self.client.get_object(Bucket=bucket, Key=key)
            self.log.info('event=get-s3-object-success bucket={} key={}'.format(bucket, key))
            return s3_object, ''
        except ClientError as e:
            if e.response['Error']['Code'] == '404':
                error_msg = 'The object key does not exist.'
            elif e.response['Error']['Code'] == '403':
                error_msg = 'An error occurred (403) when calling the HeadObject operation: Forbidden.'
            else:
                error_msg = str(e)
            self.log.error('event=get-s3-object-failure bucket={} key={} message="{}"'
                           .format(bucket, key, error_msg))
            return None, error_msg

    def put_object(self,
                   bucket: str,
                   key: str,
                   data: Union[bytes, dict, str],
                   metadata=None) -> Tuple[bool, str]:
        """
        Put object into S3 bucket
        :param bucket: S3 bucket name
        :param key: S3 Object key
        :param data: Object data
        :param metadata: Object metadata
        :return: True if it is successful, False if it encountered error
        """

        if metadata is None:
            metadata = {}

        # convert dict to json string
        if isinstance(data, dict):
            data = json.dumps(data)

        try:
            self.client.put_object(Body=data, Bucket=bucket, Key=key, Metadata=metadata)
            self.log.info('event=put-s3-object-success bucket={} key={}'.format(bucket, key))
            return True, ''
        except ClientError as e:
            error_msg = str(e)
            self.log.error('event=put-s3-object-failure bucket="{}" key="{}" message="{}"'
                           .format(bucket, key, error_msg))
        except BaseException as e:
            error_msg = str(e)
            self.log.error('event=put-s3-object-failure bucket="{}" key="{}" message="{}"'
                           .format(bucket, key, error_msg))
        return False, error_msg

    def get_metadata_from_object(self, bucket: str, key: str) -> Tuple[dict, str]:
        """
        Get metadata from S3 object
        :param bucket: S3 bucket name
        :param key: S3 Object key
        :return: Metadata of S3 object
        """
        try:
            response = self.client.head_object(Bucket=bucket, Key=key)
            self.log.info('event=get-s3-metadata-success bucket={} key={}'.format(bucket, key))
            return response.get('Metadata'), ''
        except ClientError as e:
            if e.response['Error']['Code'] == '404':
                error_msg = 'The object key does not exist.'
            else:
                error_msg = str(e)
            self.log.error('event=get-s3-metadata-failure bucket="{}" key="{}" message="{}"'
                           .format(bucket, key, error_msg))
        except BaseException as e:
            error_msg = str(e)
            self.log.error('event=get-s3-metadata-failure bucket="{}" key="{}" message="{}"'
                           .format(bucket, key, error_msg))
        return {}, error_msg



import boto3
from dsc_labs.libs.logger import DSCLabsLoggerMixin


class AWSConnector(DSCLabsLoggerMixin):
    def __init__(self, service: str, *argv, **kwargs) -> None:
        super().__init__()
        aws_access_key_id = kwargs.get('aws_access_key_id', None)
        aws_secret_access_key = kwargs.get('aws_secret_access_key', None)
        if (aws_access_key_id is None) or (aws_secret_access_key is None):
            self.client = boto3.client(service)
        else:
            self.client = boto3.client(service, *argv, **kwargs)
